/* glplib.h (low-level routines) */

/*----------------------------------------------------------------------
-- This code is part of GNU Linear Programming Kit (GLPK).
--
-- Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
-- Department for Applied Informatics, Moscow Aviation Institute,
-- Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
--
-- GLPK is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- GLPK is distributed in the hope that it will be useful, but WITHOUT
-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-- or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with GLPK; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-- 02110-1301, USA.
----------------------------------------------------------------------*/

#ifndef _GLPLIB_H
#define _GLPLIB_H

#define ulong_t               _glp_lib_ulong_t
#define uldiv_t               _glp_lib_uldiv_t

typedef struct { unsigned int lo, hi; } ulong_t;
typedef struct { ulong_t quot, rem; } uldiv_t;

typedef struct LIBENV LIBENV;
typedef struct LIBMEM LIBMEM;

#define LIB_MAX_OPEN 20
/* maximal number of simultaneously open i/o streams */

struct LIBENV
{     /* library environmental block */
      /*--------------------------------------------------------------*/
      /* memory allocation */
      LIBMEM *mem_ptr;
      /* pointer to the linked list of allocated memory blocks */
      ulong_t mem_limit;
      /* maximal amount of memory (in bytes) available for dynamic
         allocation */
      ulong_t mem_total;
      /* total amount of currently allocated memory (in bytes; is the
         sum of the size fields over all memory block descriptors) */
      ulong_t mem_tpeak;
      /* peak value of mem_total */
      ulong_t mem_count;
      /* total number of currently allocated memory blocks */
      ulong_t mem_cpeak;
      /* peak value of mem_count */
      /*--------------------------------------------------------------*/
      /* input/output streams */
      void *file_slot[LIB_MAX_OPEN]; /* FILE *file_slot[]; */
      /* file_slot[k], 0 <= k <= LIB_MAX_OPEN-1, is a pointer to k-th
         i/o stream; if k-th slot is free, file_slot[k] is NULL */
      void *log_file; /* FILE *log_file; */
      /* pointer to output stream used for hardcopying messages output
         on the screen; NULL means the hardcopy is not used */
      /*--------------------------------------------------------------*/
      /* user-defined routines */
      int (*print_hook)(void *info, char *buf);
      /* user-defined print hook routine */
      void *print_info;
      /* transit pointer passed to the routine print_hook */
      int (*fault_hook)(void *info, char *buf);
      /* user-defined fault hook routine */
      void *fault_info;
      /* transit pointer passed to the routine fault_hook */
};

#define LIB_MEM_FLAG 0x20101960
/* value used as memory block descriptor flag */

struct LIBMEM
{     /* memory block descriptor */
      int flag;
      /* descriptor flag */
      int size;
      /* size of block (in bytes, including descriptor) */
      LIBMEM *prev;
      /* pointer to the previous memory block descriptor */
      LIBMEM *next;
      /* pointer to the next memory block descriptor */
};

#define bigmul                _glp_lib_bigmul
#define bigdiv                _glp_lib_bigdiv

#define ulset                 _glp_lib_ulset
#define uladd                 _glp_lib_uladd
#define ulsub                 _glp_lib_ulsub
#define ulcmp                 _glp_lib_ulcmp
#define ulmul                 _glp_lib_ulmul
#define uldiv                 _glp_lib_uldiv

#define lib_set_ptr           _glp_lib_set_ptr
#define lib_get_ptr           _glp_lib_get_ptr

#define lib_init_env          _glp_lib_init_env
#define lib_env_ptr           _glp_lib_env_ptr
#define lib_free_env          _glp_lib_free_env
#define lib_printf            _glp_lib_printf
#define print                 _glp_lib_print             /* obsolete */
#define lib_fault             _glp_lib_fault
#define fault                 _glp_lib_fault1            /* obsolete */
#define lib_assert            _glp_lib_assert
#define lib_malloc            _glp_lib_malloc
#define lib_calloc            _glp_lib_calloc
#define lib_free              _glp_lib_free
#define umalloc               _glp_lib_malloc            /* obsolete */
#define ucalloc               _glp_lib_calloc            /* obsolete */
#define ufree                 _glp_lib_free              /* obsolete */
#define lib_fopen             _glp_lib_fopen
#define lib_fclose            _glp_lib_fclose
#define ufopen                _glp_lib_fopen             /* obsolete */
#define ufclose               _glp_lib_fclose            /* obsolete */
#define lib_open_log          _glp_lib_open_log
#define lib_close_log         _glp_lib_close_log
#define lib_print_hook        _glp_lib_print_hook
#define lib_fault_hook        _glp_lib_fault_hook

#define lib_get_time          _glp_lib_get_time
#define utime                 _glp_lib_get_time          /* obsolete */

#define str2int               _glp_lib_str2int
#define str2dbl               _glp_lib_str2dbl
#define strspx                _glp_lib_strspx
#define strtrim               _glp_lib_strtrim
#define fp2rat                _glp_lib_fp2rat

#define write_bmp16           _glp_lib_write_bmp16

void bigmul(int n, int m, unsigned short x[], unsigned short y[]);
/* multiply unsigned integer numbers of arbitrary precision */

void bigdiv(int n, int m, unsigned short x[], unsigned short y[]);
/* divide unsigned integer numbers of arbitrary precision */

ulong_t ulset(unsigned int hi, unsigned int lo);
/* construct an unsigned long integer */

ulong_t uladd(ulong_t x, ulong_t y);
/* add unsigned long integers */

ulong_t ulsub(ulong_t x, ulong_t y);
/* subtract unsigned long integers */

int ulcmp(ulong_t x, ulong_t y);
/* compare unsigned long integers */

ulong_t ulmul(ulong_t x, ulong_t y);
/* multiply unsigned long integers */

uldiv_t uldiv(ulong_t x, ulong_t y);
/* divide unsigned long integers */

void lib_set_ptr(void *ptr);
/* store global pointer in TLS */

void *lib_get_ptr(void);
/* obtain global pointer from TLS */

int lib_init_env(void);
/* initialize library environment */

LIBENV *lib_env_ptr(void);
/* obtain a pointer to the environmental block */

int lib_free_env(void);
/* free library environment */

void lib_printf(char *fmt, ...);
/* display informative message */

void print(char *fmt, ...);
/* display informative message */

void lib_fault(char *fmt, ...);
/* display error message and terminate program execution */

void fault(char *fmt, ...);
/* display error message and terminate program execution */

#define insist(expr) \
((void)((expr) || (lib_assert(#expr, __FILE__, __LINE__), 1)))

void lib_assert(char *expr, char *file, int line);
/* check for logical condition */

/* some processors need data to be properly aligned; the align_boundary
   macro defines the boundary, which should fit for all data types; the
   align_datasize macro allows enlarging size of data item in order the
   immediately following data of any type should be properly aligned */

#define align_boundary sizeof(double)

#define align_datasize(size) \
((((size) + (align_boundary - 1)) / align_boundary) * align_boundary)

void *lib_malloc(int size);
/* allocate memory block */

void *lib_calloc(int n, int size);
/* allocate memory block */

void lib_free(void *ptr);
/* free memory block */

void *lib_fopen(char *fname, char *mode);
/* open i/o stream */

void lib_fclose(void *fp);
/* close i/o stream */

int lib_open_log(char *fname);
/* open hardcopy file */

int lib_close_log(void);
/* close hardcopy file */

void lib_print_hook(int (*func)(void *info, char *buf), void *info);
/* install print hook routine */

void lib_fault_hook(int (*func)(void *info, char *buf), void *info);
/* install fault hook routine */

double lib_get_time(void);
/* determine the current universal time */

int str2int(char *str, int *val);
/* convert character string to value of integer type */

int str2dbl(char *str, double *val);
/* convert character string to value of double type */

char *strspx(char *str);
/* remove all spaces from character string */

char *strtrim(char *str);
/* remove trailing spaces from character string */

int fp2rat(double x, double eps, double *p, double *q);
/* convert floating-point number to rational number */

int write_bmp16(char *fname, int m, int n, char map[]);
/* write 16-color raster image in Windows Bitmap format */

#endif

/* eof */
