/* glplib04.c (library environment routines) */

/*----------------------------------------------------------------------
-- This code is part of GNU Linear Programming Kit (GLPK).
--
-- Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
-- Department for Applied Informatics, Moscow Aviation Institute,
-- Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
--
-- GLPK is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2, or (at your option)
-- any later version.
--
-- GLPK is distributed in the hope that it will be useful, but WITHOUT
-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-- or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with GLPK; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-- 02110-1301, USA.
----------------------------------------------------------------------*/

#undef NDEBUG
#include <assert.h>
#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "glplib.h"

/*----------------------------------------------------------------------
-- lib_init_env - initialize library environment.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- int lib_init_env(void);
--
-- DESCRIPTION
--
-- The routine lib_init_env initializes the library environment block
-- used by other low-level library routines.
--
-- This routine is called automatically on the first call some library
-- routine, so the user needn't to call this routine explicitly.
--
-- RETURNS
--
-- The routine lib_init_env returns one of the following codes:
--
-- 0 - no errors;
-- 1 - the library environment has been already initialized;
-- 2 - initialization failed (insufficient memory). */

int lib_init_env(void)
{     LIBENV *env;
      int k;
      /* check the size of some standard types */
      assert(CHAR_BIT == 8);
      assert(sizeof(char) == 1);
      assert(sizeof(short) == 2);
      assert(sizeof(int) == 4);
      /* obtain a pointer to the environmental block */
      env = lib_get_ptr();
      /* check if the environment has been already initialized */
      if (env != NULL) return 1;
      /* allocate the environmental block */
      env = malloc(sizeof(LIBENV));
      /* check if the block has been successfully allocated */
      if (env == NULL) return 2;
      /* save a pointer to the environmental block */
      lib_set_ptr(env);
      /* initialize the environmental block */
      env->mem_ptr = NULL;
      env->mem_limit = ulset(0xFFFFFFFF, 0xFFFFFFFF);
      env->mem_total = ulset(0, 0);
      env->mem_tpeak = ulset(0, 0);
      env->mem_count = ulset(0, 0);
      env->mem_cpeak = ulset(0, 0);
      for (k = 0; k < LIB_MAX_OPEN; k++) env->file_slot[k] = NULL;
      env->log_file = NULL;
      env->print_hook = NULL;
      env->print_info = NULL;
      env->fault_hook = NULL;
      env->fault_info = NULL;
      /* initialization completed */
      return 0;
}

/*----------------------------------------------------------------------
-- lib_env_ptr - obtain a pointer to the environmental block.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- LIBENV *lib_env_ptr(void);
--
-- DESCRIPTION
--
-- The routine lib_env_ptr obtains and returns a pointer to the library
-- environmental block.
--
-- If the library environment has not been initialized yet, the routine
-- performs initialization. If initialization fails, the routine prints
-- an error message to stderr and terminates the program.
--
-- RETURNS
--
-- The routine returns a pointer to the library environmental block. */

LIBENV *lib_env_ptr(void)
{     LIBENV *env;
      /* obtain a pointer to the environmental block */
      env = lib_get_ptr();
      /* check if the environment has been already initialized */
      if (env == NULL)
      {  /* not initialized yet; perform initialization */
         if (lib_init_env() != 0)
         {  /* initialization failed; display the error message */
            fprintf(stderr, "lib_env_ptr: library environment initializ"
               "ation failed\n");
            /* and abnormally terminate the program */
            abort();
         }
         /* initialization completed; obtain a pointer */
         env = lib_get_ptr();
      }
      return env;
}

/*----------------------------------------------------------------------
-- lib_free_env - free library environment.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- int lib_free_env(void);
--
-- DESCRIPTION
--
-- The routine lib_free_env frees all resources (memory blocks, etc.),
-- which were allocated by the library routines and which are currently
-- still in use.
--
-- The user does not need to call this routine until he would like to
-- explicitly free all the resources.
--
-- RETURNS
--
-- 0 - no errors;
-- 1 - the library environment is inactive (not initialized). */

int lib_free_env(void)
{     LIBENV *env;
      int k;
      /* retrieve a pointer to the environmental block */
      env = lib_get_ptr();
      /* check if the environment is active */
      if (env == NULL) return 1;
      /* free memory blocks, which are still allocated */
      while (env->mem_ptr != NULL)
      {  LIBMEM *blk = env->mem_ptr;
         env->mem_ptr = blk->next;
         free(blk);
      }
      /* close i/o streams, which are still open */
      for (k = 0; k < LIB_MAX_OPEN; k++)
         if (env->file_slot[k] != NULL) fclose(env->file_slot[k]);
      /* free memory allocated to the environmental block */
      free(env);
      /* reset a pointer to the environmental block */
      lib_set_ptr(NULL);
      /* deinitialization completed */
      return 0;
}

/*----------------------------------------------------------------------
-- lib_printf - display informative message.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void lib_printf(char *fmt, ...);
--
-- DESCRIPTION
--
-- The routine lib_printf formats an informative message specified by
-- the format control string fmt and optional parameter list, and writes
-- it to the text device. */

void lib_printf(char *fmt, ...)
{     LIBENV *env = lib_env_ptr();
      va_list arg;
      char buf[4095+1];
      /* format the message */
      va_start(arg, fmt);
      vsprintf(buf, fmt, arg);
      insist(strlen(buf) < sizeof(buf));
      va_end(arg);
      /* pass the message to the user-defined routine */
      if (env->print_hook != NULL)
         if (env->print_hook(env->print_info, buf) != 0) goto skip;
      /* send the message to the text device */
      fputs(buf, stdout);
      /* write the message to the hardcopy file */
      if (env->log_file != NULL)
         fputs(buf, env->log_file);
skip: return;
}

void print(char *fmt, ...)
{     /* obsolete version */
      va_list arg;
      char msg[4095+1];
      va_start(arg, fmt);
      vsprintf(msg, fmt, arg);
      insist(strlen(msg) <= 4095);
      va_end(arg);
      lib_printf("%s\n", msg);
      return;
}

/*----------------------------------------------------------------------
-- lib_fault - display error message and terminate program execution.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void lib_fault(char *fmt, ...);
--
-- DESCRIPTION
--
-- The routine fault formats an error message specified by the format
-- control string fmt and optional parameter list, writes it to the text
-- device, and then terminates execution of the program. */

void lib_fault(char *fmt, ...)
{     LIBENV *env = lib_env_ptr();
      va_list arg;
      char buf[4095+1];
      /* format the message */
      va_start(arg, fmt);
      vsprintf(buf, fmt, arg);
      insist(strlen(buf) < sizeof(buf));
      va_end(arg);
      /* pass the message to the user-defined routine */
      if (env->fault_hook != NULL)
         if (env->fault_hook(env->fault_info, buf) != 0) goto skip;
      /* send the message to the text device */
      fputs(buf, stdout);
      /* write the message to the hardcopy file */
      if (env->log_file != NULL)
         fputs(buf, env->log_file);
skip: /* terminate program execution */
      abort();
      /* no return */
}

void fault(char *fmt, ...)
{     /* obsolete version */
      va_list arg;
      char msg[4095+1];
      va_start(arg, fmt);
      vsprintf(msg, fmt, arg);
      insist(strlen(msg) <= 4095);
      va_end(arg);
      lib_fault("%s\n", msg);
      /* no return */
}

/*----------------------------------------------------------------------
-- insist - check for logical condition.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void insist(int expr);
--
-- DESCRIPTION
--
-- The routine insist (implemented as a macro) checks for a logical
-- condition specified by the parameter expr. If the condition is false
-- (i.e. expr is zero), the routine displays an error message and then
-- abnormally terminates the program.
--
-- This routine is a replacement of the standard function assert. */

void lib_assert(char *expr, char *file, int line)
{     fault("Assertion failed: %s; file %s; line %d", expr, file, line);
      /* no return */
}

/*----------------------------------------------------------------------
-- lib_malloc - allocate memory block.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void *lib_malloc(int size);
--
-- DESCRIPTION
--
-- The routine lib_malloc allocates a memory block of size bytes long.
--
-- Note that being allocated the memory block contains arbitrary data
-- (not binary zeros).
--
-- RETURNS
--
-- The routine lib_malloc returns a pointer to the allocated memory
-- block. To free this block the routine lib_free (not free!) should be
-- used. */

void *lib_malloc(int size)
{     LIBENV *env = lib_env_ptr();
      LIBMEM *desc;
      int size_of_desc = align_datasize(sizeof(LIBMEM));
      if (size < 1)
         fault("lib_malloc: size = %d; invalid parameter", size);
      if (size > INT_MAX - size_of_desc)
         fault("lib_malloc: size = %d; size too big", size);
      size += size_of_desc;
      if (ulcmp(ulset(0, size), ulsub(env->mem_limit, env->mem_total))
         > 0)
         fault("lib_malloc: size = %d; memory limit exceeded", size);
      desc = malloc(size);
      if (desc == NULL)
         fault("lib_malloc: size = %d; no memory available", size);
      memset(desc, '?', size);
      desc->size = size;
      desc->flag = LIB_MEM_FLAG;
      desc->prev = NULL;
      desc->next = env->mem_ptr;
      if (desc->next != NULL) desc->next->prev = desc;
      env->mem_ptr = desc;
      env->mem_total = uladd(env->mem_total, ulset(0, size));
      if (ulcmp(env->mem_tpeak, env->mem_total) < 0)
         env->mem_tpeak = env->mem_total;
      env->mem_count = uladd(env->mem_count, ulset(0, 1));
      if (ulcmp(env->mem_cpeak, env->mem_count) < 0)
         env->mem_cpeak = env->mem_count;
      return (void *)((char *)desc + size_of_desc);
}

/*----------------------------------------------------------------------
-- lib_calloc - allocate memory block.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void *lib_calloc(int n, int size);
--
-- DESCRIPTION
--
-- The routine lib_calloc allocates a memory block of (n*size) bytes
-- long.
--
-- Note that being allocated the memory block contains arbitrary data
-- (not binary zeros).
--
-- RETURNS
--
-- The routine lib_calloc returns a pointer to the allocated memory
-- block. To free this block the routine lib_free (not free!) should be
-- used. */

void *lib_calloc(int n, int size)
{     if (n < 1)
         fault("lib_calloc: n = %d; invalid parameter", n);
      if (size < 1)
         fault("lib_calloc: size = %d; invalid parameter", size);
      if (n > INT_MAX / size)
         fault("lib_calloc: n = %d; size = %d; array too big", n, size);
      return umalloc(n * size);
}

/*----------------------------------------------------------------------
-- lib_free - free memory block.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void lib_free(void *ptr);
--
-- DESCRIPTION
--
-- The routine lib_free frees the memory block which is pointed to by
-- ptr and which was previuosly allocated by the routine lib_malloc or
-- lib_calloc. */

void lib_free(void *ptr)
{     LIBENV *env = lib_env_ptr();
      LIBMEM *desc;
      int size_of_desc = align_datasize(sizeof(LIBMEM));
      if (ptr == NULL)
         fault("lib_free: ptr = %p; null pointer", ptr);
      desc = (void *)((char *)ptr - size_of_desc);
      if (desc->flag != LIB_MEM_FLAG)
         fault("lib_free: ptr = %p; invalid pointer", ptr);
      if (ulcmp(env->mem_total, ulset(0, desc->size)) < 0 ||
          ulcmp(env->mem_count, ulset(0, 0)) == 0)
         fault("lib_free: ptr = %p; memory allocation error", ptr);
      if (desc->prev == NULL)
         env->mem_ptr = desc->next;
      else
         desc->prev->next = desc->next;
      if (desc->next == NULL)
         ;
      else
         desc->next->prev = desc->prev;
      env->mem_total = ulsub(env->mem_total, ulset(0, desc->size));
      env->mem_count = ulsub(env->mem_count, ulset(0, 1));
      memset(desc, '?', size_of_desc);
      free(desc);
      return;
}

/*----------------------------------------------------------------------
-- lib_fopen - open i/o stream.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void *lib_fopen(char *fname, char *mode);
--
-- DESCRIPTION
--
-- The routine lib_fopen opens a file using the character string fname
-- as the file name and the character string mode as the open mode.
--
-- RETURNS
--
-- If the file has been open, the routine lib_fopen returns a pointer
-- to the i/o stream associated with the file (i.e. a pointer to object
-- of FILE type). Otherwise the routine return NULL. */

void *lib_fopen(char *fname, char *mode)
{     LIBENV *env = lib_env_ptr();
      int k;
      /* find free slot */
      for (k = 0; k < LIB_MAX_OPEN; k++)
         if (env->file_slot[k] == NULL) break;
      if (k == LIB_MAX_OPEN)
         fault("lib_fopen: too many open files");
      /* open a file and store a pointer to the i/o stream */
      env->file_slot[k] = fopen(fname, mode);
      return env->file_slot[k];
}

/*----------------------------------------------------------------------
-- lib_fclose - close i/o stream.
--
-- SYNOPSIS
--
-- #include "glplib.h"
-- void lib_fclose(void *fp);
--
-- DESCRIPTION
--
-- The routine lib_fclose closes a file associated with an i/o stream,
-- which the parameter fp points to. It is assumed that the stream was
-- open by the routine lib_fopen. */

void lib_fclose(void *fp)
{     LIBENV *env = lib_env_ptr();
      int k;
      /* check if the i/o stream pointer is valid */
      if (fp == NULL)
         fault("lib_fclose: fp = %p; null i/o stream", fp);
      for (k = 0; k < LIB_MAX_OPEN; k++)
         if (env->file_slot[k] == fp) break;
      if (k == LIB_MAX_OPEN)
         fault("lib_fclose: fp = %p; invalid i/o stream", fp);
      /* close a file and free the corresponding slot */
      fclose(fp);
      env->file_slot[k] = NULL;
      return;
}

int lib_open_log(char *fname)
{     /* open hardcopy file */
      LIBENV *env = lib_env_ptr();
      if (env->log_file != NULL)
      {  /* hardcopy file is already open */
         return 1;
      }
      env->log_file = ufopen(fname, "w");
      if (env->log_file == NULL)
      {  /* cannot create hardcopy file */
         return 2;
      }
      setvbuf(env->log_file, NULL, _IOLBF, BUFSIZ);
      return 0;
}

int lib_close_log(void)
{     /* close hardcopy file */
      LIBENV *env = lib_env_ptr();
      if (env->log_file == NULL)
      {  /* hardcopy file is already closed */
         return 1;
      }
      ufclose(env->log_file);
      env->log_file = NULL;
      return 0;
}

void lib_print_hook(int (*func)(void *info, char *buf), void *info)
{     /* install print hook routine */
      LIBENV *env = lib_env_ptr();
      env->print_hook = func;
      env->print_info = info;
      return;
}

void lib_fault_hook(int (*func)(void *info, char *buf), void *info)
{     /* install fault hook routine */
      LIBENV *env = lib_env_ptr();
      env->fault_hook = func;
      env->fault_info = info;
      return;
}

/* eof */
